/**
 * @constructor
 */
function AdController() {
	
	window.adBreakPlaylist = [];
	
	this.publisherAPI = null;
	this.facade = null;
	
	// You can check the global default values by calling
	// `this.globalConfig = smartclientcore.SCHEMA.create(smartclientcore.SCHEMA.GLOBAL_CONFIG);`
	
	// Though, it is sufficient to specify only properties that override the default values.
	this.globalConfig = {
		debug: false,
		timeouts: {
			adRequest: 6,
			adRequestChain: 12,
			adSlotRequest: 24,
			videoRequest: 18,
			vpaidRequest: 6,
			vpaidReadiness: 18
		},
		
		// Most SmartTV do not support XHR requests. Chose JSONP instead.
		adRequestMethod: smartclientcore.AD_REQUEST_METHOD.JSONP
	};
	
	// Used for passing implementation-specific runtime variables.
	//  For a full list of properties please consult `vendor_modules/smartclip/docs/setup.md#environmentvars`
	this.environmentVars = {
		desiredBitrate: 1000
	};
	
	// For testing purpose: Definition of common ad breaks and their execution time.
	// 	{number} relativeOffset - The expected execution time between start (0) and end (1) of the content video.
	// 	{number} actualOffset - The offset in seconds based on the duration of the content video. (calculated at runtime)
	//  {AdBreak} adBreakSetup - Optional configuration to override the default adBreak as returned from `this.getDefaultAdBreak()` below.
	this.testAdBreak = {
		'preroll': {
			relativeOffset: 0,
			actualOffset: 0
		},
		'midroll': {
			relativeOffset: 0.5,
			actualOffset: -1
		},
		'capped': {
			relativeOffset: 0.5,
			actualOffset: -1,
			adBreakSetup:{
				tag: 'https://example.com'
			}
		},
		'postroll': {
			relativeOffset: 1,
			actualOffset: -1
		}
	};
}

/**
 * Wraps a custom Video Player API into a common Video Player Facade that is know to the smartclientcore.
 * @param {*|Object} player - the reference to the PlayerAPI
 * @returns {void}
 */
AdController.prototype.createPlayerFacade = function (player) {
	this.facade = new CEHtmlFacade(player);
};

/**
 * Connects the video player and advertising logic. As a result we get the PublisherAPI for the current session.
 * Also see `vendor_modules/smartclip/docs/plugin-static-api.md`
 * @returns {void}
 */
AdController.prototype.initSmartClientCore = function () {
	var that = this;
	
	window.smartclientcore.connect(this.facade).setup(this.globalConfig).then(function (api) {
		
		// This is our advertising interface for the current session...
		//  API specs: `vendor_modules/smartclip/docs/PublisherAPI.md`
		that.publisherAPI = api;
		
		// ... and a good point to subscribe to ad events.
		that.initAdEventListeners();
		
		// The smartclientcore depends on player updates that are comparable to HTML Media Events.
		//  Since the CE-HTML video object doesn't dispatch any events but only calls a common `onPlayStateChange` handler,
		//  all changes must be translated into the corresponding event.
		that.facade.initPlayerStateListener();
		
		writeLog({
			type: 'Info',
			timestamp: new Date(),
			text: 'smartclientcore initialization done.'
		});
		
	}, function (failure) {
		
		writeLog({
			type: 'Error',
			timestamp: new Date(),
			text: 'smartclientcore initialization failed:' + failure
		});
	});
};

/**
 * Subscribes to smartclientcore AdEvents and calls the `handleAdEvent` callback below.
 * Event list and description `vendor_modules/smartclip/docs/events.md`
 * @returns {void}
 */
AdController.prototype.initAdEventListeners = function () {
	var that = this;
	
	Object.keys(window.smartclientcore.EVENT).forEach(function (key) {
		that.publisherAPI.addEventListener(window.smartclientcore.EVENT[key], function (event) {
			that.handleAdEvent(event);
		});
	});
};

/**
 * Invokes actions based on the received AdEvent.
 * @param {AdEvent} event triggered by the PublisherAPI
 * @returns {void}
 */
AdController.prototype.handleAdEvent = function (event) {
	
	writeLog(event);
	
	if (event.type === window.smartclientcore.EVENT.ON_AD_STARTED) {
		this.facade.adIsRunnig = true;
		
		// Force ad playback
		window.playerAPI.play(1);
	}
	if (event.type === window.smartclientcore.EVENT.ON_AD_FINISHED || event.type === window.smartclientcore.EVENT.ON_AD_SKIPPED) {
		this.facade.adIsRunnig = false;
	}
	if (event.type === window.smartclientcore.EVENT.ON_AD_SLOT_FINISHED) {
		this.facade.adIsRunnig = false;
	}
};

/**
 * Invokes a commercial break by requesting the specified ad setup from the ad server.
 * @param {AdBreak} adBreak - Configuration for the upcoming ad slot.
 * @returns {void}
 */
AdController.prototype.requestAdSlot = function (adBreak) {
	var that = this;
	
	that.publisherAPI.initAdSlot(adBreak, that.environmentVars).then(function () {
		try {
			that.publisherAPI.startAdSlot();
			console.log('[AdController] - OK - startAdSlot');
		} catch (e) {
			console.log('[AdController] - FAIL - startAdSlot', e);
		}
	}, function (failure) {
		console.log('[AdController] - FAIL - requestAdSlot', failure);
	});
};

/**
 * Creates a new standard AdBreak definition and returns it to the caller.
 * @returns {AdBreak} Default AdBreak to request a list of ads from the ad-server (AdPod),
 *  that is surrounded with branded separation clips (Opener, Bumper, Closer)
 */
AdController.prototype.getDefaultAdBreak = function () {
	return {
		tag: 'http://ad.sxp.smartclip.net/select?type=dyn&ple=testcors.eu.smartclip~~400x320&cu_test=vast3&fwd_sz=400x320&rnd=[CACHEBUSTING]',
		opener: {
			progressive: 'http://cdn.smartclip.net/assets/testads/Smartclip_Opener_1080p.mp4?' + Math.random() * 10000,
			forcePlay: true
		},
		closer: {
			progressive: 'http://cdn.smartclip.net/assets/testads/Smartclip_Closer_1080p.mp4?' + Math.random() * 10000,
			forcePlay: true
		},
		bumper: {
			progressive: 'http://cdn.smartclip.net/assets/testads/Smartclip_Bumper_1080p.mp4?' + Math.random() * 10000,
			forcePlay: false
		}
	};
};

/**
 * Defines the commercial breaks for the upcoming content video.
 * @param {Array} schedule - list of test cases
 * @returns {void}
 */
AdController.prototype.applyAdSchedule = function (schedule) {
	
	// reset schedule before setting the new one
	window.adBreakPlaylist = [];
	
	for (var i = 0; i < schedule.length; i++) {
		
		var adBreakName = schedule[i];
		var adBreakSetup = this.testAdBreak[adBreakName];
		
		// Calculate the execution time based on the current content duration
		adBreakSetup.actualOffset = (window.playerAPI.playTime || 210) * adBreakSetup.relativeOffset;
		adBreakSetup.adBreak = adBreakSetup.adBreak || this.getDefaultAdBreak();
		
		console.log('[AdController] - Schedule AdBreak ' + adBreakName + '@' + adBreakSetup.actualOffset);
		window.adBreakPlaylist.push(adBreakSetup);
	}
};

AdController.prototype.enableDebug = function() {
	
	// when enabling smartclientcore debug the console.table method needs
	// to be mapped to console.log, as most TV's do no support console.table
	console.table = console.log;
	
	window.localStorage.setItem('debug', true);
};

var module = module || {};
module.adController = AdController;
