function CEHtmlFacade(playerReference) {

	this.player = playerReference;
	this.currentContentSource = null;

	this.adIsRunnig = false;

	this.timer = null;

	/**
	 * Map player specific events to the corresponding HTML5 MediaEvent
	 */
	this.PLAYER_EVENT_MAP = {
		"ended": "ended",
		"error": "error",
		"loadeddata": "loadeddata",
		"timeupdate": "timeupdate",
		"playerPlay": "playing",
		"playerPause": "pause",
		"volumeChanged": "volumeChanged",
		"muted": "muted",
		"unmuted": "unmuted"
	};

	this.PLAYER_EVENT_CALLBACK_MAP = {
		"ended": function () {},
		"error": function () {},
		"loadeddata": function () {},
		"timeupdate": function () {},
		"playerPlay": function () {},
		"playerPause": function () {},
		"volumeChanged": function () {},
		"muted": function () {},
		"unmuted": function () {},
	};
}

// --- Custom facade interface methods ---

/**
 * Due to the absence of HTML5 MediaEvents on SmartTVs, we have to use the video object playstatechange callback
 * and trigger the corresponding MediaEvent accordingly.
 * @returns {void}
 */
CEHtmlFacade.prototype.initPlayerStateListener = function () {
	
	// Use `that` to keep a reference to the local scope.
	// Within the callback `this` refers to the global scope.
	var that = this;
	
	that.player.onPlayStateChange = function () {
		switch (that.player.playState) {
			case 3:
				// connecting
				console.log('[CEHTMLFacade] - playState = connecting');
				that.PLAYER_EVENT_CALLBACK_MAP['loadeddata']();
				break;
			case 1:
				// playing
				console.log('[CEHTMLFacade] - playState = playing');
				that.PLAYER_EVENT_CALLBACK_MAP['playerPlay']();
				that.timer = setInterval(function () {
					if (that.adIsRunnig) {
						that.PLAYER_EVENT_CALLBACK_MAP['timeupdate']();
					} else {
						window.adBreakPlaylist.forEach(function(playlistItem, index) {
							if (that.player.playPosition > playlistItem.actualOffset) {
								console.log('[CEHTMLFacade] - requestAdSlot from Tests at "' + that.player.playPosition + '" > "' + playlistItem.actualOffset + '" @ i: "' + index + '"');
								adController.requestAdSlot(playlistItem.adBreak);
								window.adBreakPlaylist.splice(index, 1);
							}
						});
					}
				}, 250);
				break;
			case 2:
				// paused
				console.log('[CEHTMLFacade] - playState = paused');
				if (that.timer) {
					clearInterval(that.timer);
					that.timer = null;
				}
				break;
			case 0:
				// stopped
				console.log('[CEHTMLFacade] - playState = stopped, adIsRunnig = ' + that.adIsRunnig);
				
				// Avoid sending the `ended` event before the `loadedmetadata` (AdStarted).
				if (that.adIsRunnig) {
					that.PLAYER_EVENT_CALLBACK_MAP['ended']();
				}
				if (that.timer) {
					clearInterval(that.timer);
					that.timer = null;
				}
				break;
			case 4:
				// buffering
				console.log('[CEHTMLFacade] - playState = buffering');
				if (that.timer) {
					clearInterval(that.timer);
					that.timer = null;
				}
				break;
			case 5:
				// finished
				console.log('[CEHTMLFacade] - playState = finished');
				that.PLAYER_EVENT_CALLBACK_MAP['ended']();
				if (that.timer) {
					clearInterval(that.timer);
					that.timer = null;
				}
				break;
			case 6:
				// error
				console.log('[CEHTMLFacade] - playState = ERROR', that.player.error);
				that.PLAYER_EVENT_CALLBACK_MAP['error']();
				break;
		}
	};
};

// --- Common facade interface methods as defined in vendor_modules/smartclip/FacadeBase.js ---

/**
 * Requests the player facade version (API version) to ensure the particular versions of `smartclientcore` and player facade
 * are supported.
 * @param {string} apiFacadeVersion - Required API version
 * @returns {string} Supported API version
 */
CEHtmlFacade.prototype.handshakeVersion = function (apiFacadeVersion) {
	return "3.0";
};

/**
 * Requests the video player to load and play the specified media file.
 * @param {string} url - Media file URI
 * @param {Boolean} autoPlay - Whether the requested media file starts playback immediately.
 * @param {Boolean} disableSeeking - Whether the requested source can be seeked back and forth.
 * @returns {Promise} Resolves if the player accepted the media file request
 */
CEHtmlFacade.prototype.load = function (url, autoPlay, disableSeeking) {
	
	var player = this.player;
	
	var _autoPlay = autoPlay === undefined ? true : autoPlay;
	var _disableSeeking = disableSeeking === undefined ? false : disableSeeking;
	
	var _url = url;
	if (_url.indexOf("//") === 0) {
		_url = "https:" + url;
	}
	console.log('[CEHTMLFacade] - prototype.load: "' + _url + '" - autoplay: "' + (_autoPlay ? 'YES' : 'NO') + '" - seeking: "' + (_disableSeeking ? 'UNALLOWED' : 'ALLOWED') + '"');
	return new Promise(function (resolve, reject) {
		try {
			
			//that.addEventListener('loadeddata', function() {
			//	console.log('[CEHTMLFacade] - load request resolved.');
			//  resolve();
			//});
			
			player.data = _url;
			resolve();
		} catch (error) {
			console.log('[CEHTMLFacade] - load request failed.', error);
			reject();
		}
	});
};

/**
 * Sets up a function that will be called whenever the specified event is delivered to the video player.
 * @param {string} type - A case-sensitive string representing the player event type to listen for.
 * @param {Function} callback - The function which receives a notification when an event of the specified type occurs.
 */
CEHtmlFacade.prototype.addEventListener = function (type, callback) {
	try {
		this.PLAYER_EVENT_CALLBACK_MAP[type] = callback;
	} catch (error) {
		return NaN;
	}
};

/**
 * Removes an event listener from the video player that has been registered with `addEventListener()`
 * _*Note:* The event listener to be removed must match with the listener that has been added._
 * @param {string} type - A case-sensitive string representing the player event type to listen for.
 * @param {Function} callback - The function which receives a notification when an event of the specified type occurs.
 */
CEHtmlFacade.prototype.removeEventListener = function (type, callback) {
	try {
		this.PLAYER_EVENT_CALLBACK_MAP[type] = function () {
		};
	} catch (error) {
		return NaN;
	}
};

/**
 * @returns {SourceObject} Current content source and playhead progress
 */
CEHtmlFacade.prototype.getCurrentContentSource = function () {
	try {
		return this.currentContentSource;
	} catch (error) {
		return NaN;
	}
};

/**
 * Should freeze the current playback state (for later use with `releaseContent()`)
 * and respond to the PublisherAPI once finished.
 * @returns {Promise} Current content source and playhead progress for VAST tracking enrichment
 */
CEHtmlFacade.prototype.lockContent = function () {
	var that = this;

	return new Promise(function (resolve, reject) {
		try {
			console.log('[CEHTMLFacade] - DONE - lockContent', that.player.data);
			if (that.player.data) {
				that.currentContentSource = {
					src: that.player.data,
					currentTime: that.player.playPosition
				};
				resolve(that.currentContentSource);
			}
		} catch (error) {
			console.log('[CEHTMLFacade] - FAIL - lockContent', error);
			reject();
		}
	});
};

/**
 * Should restore the last known playback state, resume playback
 * and respond to the PublisherAPI once finished.
 * @returns {Promise} Either resolves if the player accepted the play request or not
 */
CEHtmlFacade.prototype.releaseContent = function () {
	var that = this;

	return new Promise(function (resolve, reject) {
		try {
			that.load(that.currentContentSource.src).then(function () {
				that.player.seek(that.currentContentSource.currentTime);
				that.player.play(1);
				console.log('[CEHTMLFacade] - DONE - releaseContent - seek: "' + that.currentContentSource.currentTime + '" - playPosition: "' + that.player.playPosition + '"', that.player.data);

				resolve();
			});
		} catch (error) {
			console.log('[CEHTMLFacade] - FAIL - releaseContent', error);
			reject();
		}
	});
};

/**
 * Requests details about the most recent error that has occurred on the video player.
 * @returns {MediaError} Description object of the most recent error or `null`.
 */
CEHtmlFacade.prototype.getError = function () {
	var errorMessage = '';
	switch (this.player.error) {
		// 0-2 are defined in CEA-2014A
		case 0:
			errorMessage = "A/V format not supported";
			break;
		case 1:
			errorMessage = "cannot connect to server or lost connection";
			break;
		case 2:
			errorMessage = "unidentified error";
			break;
		// 3-6 are defined in OIPF annex B
		case 3:
			errorMessage = "insufficient resources.";
			break;
		case 4:
			errorMessage = "content corrupt or invalid.";
			break;
		case 5:
			errorMessage = "content not available.";
			break;
		case 6:
			errorMessage = "content not available at given position.";
			break;
		// defined in HbbTV for download content
		case 7:
			errorMessage = "content blocked due to parental control.";
			break;
	}
	return {
		code: this.player.error,
		message: errorMessage
	};
};

/**
 * Requests the current playhead position from the video player.
 * @returns {Number} Current time in seconds.
 */
CEHtmlFacade.prototype.getCurrentTime = function () {
	try {
		return this.player.playPosition / 1000;
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the content-duration from the video player.
 * @returns {Number} Total time in seconds.
 */
CEHtmlFacade.prototype.getDuration = function () {
	try {
		return this.player.playTime / 1000;
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the current size and viewMode from the video player.
 * @returns {ElementSize} Current player dimension specs
 */
CEHtmlFacade.prototype.getPlayerSize = function () {
	return {
		boundingRect: this.player.getBoundingClientRect() || NaN,
		viewMode: window.fullScreen ? "fullscreen" : "normal"
	};
};

/**
 * Requests the current size and viewMode from the viewport that surrounds the video player.
 * @returns {ElementSize} The dimension specs of the element
 */
CEHtmlFacade.prototype.getViewportSize = function () {
	return {
		boundingRect: document.querySelector('body').getBoundingClientRect() || NaN,
		viewMode: window.fullScreen ? 'fullscreen' : 'normal'
	};
};

CEHtmlFacade.prototype.getVideoElement = function () {
	try {
		return this.player;
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the video player to change the volume.
 * @param {Number} volume - Requested volume level between 0-1
 * @returns {Promise} Resolves if the video player accepted the volume change request
 */
CEHtmlFacade.prototype.setVolume = function (volume) {
	// try {
	// 	that.player.volume = volume;
	// } catch (error) {
	// 	return NaN;
	// }
};

/**
 * Requests the current volume level from the video player.
 * @returns {Number} The current volume level between 0-1
 */
CEHtmlFacade.prototype.getVolume = function () {
	// try {
	// 	return that.player.volume;
	// } catch (error) {
	// 	return NaN;
	// }
};

/**
 * Requests the current muted state from the video player.
 * @returns {Boolean} Muted state
 */
CEHtmlFacade.prototype.isMuted = function () {
	// try {
	// 	return that.player.muted;
	// } catch (error) {
	// 	return NaN;
	// }
};

/**
 * Requests the video player to mute playback.
 * @returns {Promise} Resolves if the video player has been muted
 */
CEHtmlFacade.prototype.mute = function () {
	// try {
	// 	return that.player.muted = true;
	// } catch (error) {
	// 	return NaN;
	// }
};

/**
 * Requests the video player to unmute playback.
 * @returns {Promise} Resolves if the video player has been unmuted
 */
CEHtmlFacade.prototype.unmute = function () {
	// try {
	// 	return that.player.muted = false;
	// } catch (error) {
	// 	return NaN;
	// }
};

/**
 * Requests the current play state from the video player.
 * @returns {Boolean} Playback state
 */
CEHtmlFacade.prototype.isPaused = function () {
	try {
		return this.player.playState === 2;
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the video player to pause playback.
 * @returns {Promise} Resolves if the video player has been paused
 */
CEHtmlFacade.prototype.pause = function () {
	try {
		return this.player.play(0);
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the video player to start (resume) playback.
 * @returns {Promise} Resolves if the video player has been resumed
 */
CEHtmlFacade.prototype.play = function () {
	try {
		return this.player.play(1);
	} catch (error) {
		return NaN;
	}
};

/**
 * Requests the video player to change the playhead position to the specified time.
 * @param {Number} offset - Requested playhead position
 * @returns {Promise} Resolves if the video player has finished seeking to the specified position
 */
CEHtmlFacade.prototype.seek = function (offset) {
	try {
		return this.player.seek(offset);
	} catch (error) {
		return NaN;
	}
};

var module = module || {};
module.ceHtmlFacade = CEHtmlFacade;
